@@ -16,7 +16,7 @@
 # 02110-1301, USA.
 
 import os
-from trosnoth.data import sprites
+from trosnoth.src.trosnothgui import defines
 
 user = object() # Used to write/read user data.
 
@@ -35,14 +35,6 @@ def makeDirs(path):
     if not os.path.exists(path):
         os.makedirs(path)
 
-def loadSprite(filename):
-    import pygame
-    return pygame.image.load(getPath(sprites, filename)).convert()
-            
-def loadSpriteWithAlpha(filename):
-    import pygame
-    return pygame.image.load(getPath(sprites, filename)).convert_alpha()
-
 # Check whether this is within a py2exe zip
 def get_exe_dir():
     import imp, sys
