@@ -14,8 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
-
-from trosnoth.src.utils import logging
+from trosnoth.src.utils.utils import timeNow, RunningAverage
 
 class Element(object):
     active = True
@@ -23,6 +22,7 @@ class Element(object):
 
     def __init__(self, app):
         self.app = app
+        self.drawTime = RunningAverage(500)
     
     def processEvent(self, event):
         '''Processes the specified event and returns the event if it should
@@ -50,19 +50,12 @@ class CompoundElement(Element):
         self.elements = []
     
     def processEvent(self, event):
-        for i in range(1, 1+len(self.elements)):
-            try:
-                event = self.elements[-i].processEvent(event)
-            except ValueError:
-                # TODO: Figure out the best way of handling this sort of thing.
-                print "Warning: An element was removed from the list while " +\
-                "event-processing. This can cause events to be left " +\
-                "unprocessed; though is unlikely to have done any damage"
+        elements = list(self.elements)
+        elements.reverse()
+        for element in elements:
+            event = element.processEvent(event)
 
-            if not self.app._running:
-                return None
-
-            if event is None:
+            if event is None or not self.app._running:
                 return None
         return event
 
@@ -77,11 +70,22 @@ class CompoundElement(Element):
                 print 'ERROR: Infinite loop of elements:'
                 if hasattr(element, 'elements'):
                     print element, 'contains', element.elements
-            raise RuntimeError, e
+            raise
 
     def draw(self, screen):
         for element in self.elements:
+            t = timeNow()
             element.draw(screen)
+            element.drawTime.noteValue(timeNow() - t)
+
+    def getSlowestChild(self):
+        result = None
+        t = 0
+        for element in self.elements:
+            if element.drawTime.mean >= t:
+                result = element
+                t = element.drawTime.mean
+        return result
 
     def clearFocus(self):
         self.setFocus(None)
