@@ -28,10 +28,10 @@ class Interface(framework.CompoundElement):
         x, y = backdrop.get_size()
         size = Size(x, y)
         sc1 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(20,20), size, ScaledSize(200,200))
-        sc2 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(230,20), size, ScaledSize(200,y+scrollableCanvas.ScrollableCanvas.ScrollBar.defaultWidth))
+        sc2 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(230,20), size, ScaledSize(200,y+scrollableCanvas.ScrollBar.defaultWidth))
         sc3 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(440,20), size, ScaledSize(x,y))
-        sc4 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(20,530), size, ScaledSize(x+scrollableCanvas.ScrollableCanvas.ScrollBar.defaultWidth, 200))
-        #sc5 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(20,530), size, ScaledSize(x+scrollableCanvas.ScrollableCanvas.ScrollBar.defaultWidth, y))
+        sc4 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(20,530), size, ScaledSize(x+scrollableCanvas.ScrollBar.defaultWidth, 200))
+        #sc5 = scrollableCanvas.ScrollableCanvas(app, ScaledLocation(20,530), size, ScaledSize(x+scrollableCanvas.ScrollBar.defaultWidth, y))
         self.elements = [sc1, sc2, sc3, sc4]#, sc5]
         font = Font("KLEPTOCR.TTF", 30)
         onclick = lambda sender: self.app.screenManager.setScreenProperties((1680, 1050), 0, "Testing")
