@@ -15,18 +15,17 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-from trosnoth.data import loadSprite
-
-##
-# Takes a section of text, a font with which it will be rendered, and the
-# maximum size that a line can be, and returns a list of the new strings.
-# @param text The text to split
-# @param font The font that the text will be rendered with
-# @param width The maximum width (in pixels) of a single line of text
-# @param breakWords If this is set to true, words will be allowed to be broken
-#                   in half if it may look neater (annotated with a hyphen).
-#                   If not, words will only break if absolutely necessary
 def WordWrap(app, text, font, width, breakWords = True):
+    '''
+    Takes a section of text, a font with which it will be rendered, and the
+    maximum size that a line can be, and returns a list of the new strings.
+    @param text The text to split
+    @param font The font that the text will be rendered with
+    @param width The maximum width (in pixels) of a single line of text
+    @param breakWords If this is set to true, words will be allowed to be
+            broken in half if it may look neater (annotated with a hyphen).
+            If not, words will only break if absolutely necessary.
+    '''
     lines = text.split('\n')
     # Determine if the line needs word-wrapping:
     x = 0
@@ -65,14 +64,3 @@ def WordWrap(app, text, font, width, breakWords = True):
                 break
         x += 1
     return lines
-
-def makeImage(string, colorkey = (255,255,255)):
-    img = loadSprite(string)
-    img.set_colorkey(colorkey)
-    return img
-
-def makeImages(strings, colorkey = (255,255,255)):
-    images = []
-    for string in strings:
-        images.append(makeImage(string, colorkey))
-    return images
\ No newline at end of file
