@@ -40,6 +40,7 @@ class GrenadeShot(Unit):
         self.player = player
         self.id = id
         self.local = False
+        self.registeredHit = False
         
         # Place myself.
         self.pos = player.pos
@@ -76,15 +77,10 @@ class GrenadeShot(Unit):
         block.addGrenade(self)
               
     
-    def movedBy(self, deltaX, deltaY):
-        # Called by moveUnit().
-        pass
-
     def update(self, deltaT):
         '''Called by this player's universe when this player should update
         its position. deltaT is the time that's passed since its state was
         current, measured in seconds.'''
-        slow = False
 
         try:
             deltaX = self.xVel * deltaT
@@ -136,47 +132,34 @@ class GrenadeShot(Unit):
 
 
 class Shot(Unit):
-    '''Represents a shot fired by a player. Shots are created by the universe
-    object when it receives word from the networkClient object.'''
     
+    NORMAL = 'normal'
+    TURRET = 'turret'
+    RICOCHET = 'ricochet'
+
     # Speed that shots travel at.
-    speed = 600       # pix/s
-    lifetime = 1.    # s
+    SPEED = 600       # pix/s
+    LIFETIME = 1.     # s
     
-    def __init__(self, id, team, pos, angle, player, turret, ricochet):
+    def __init__(self, world, id, team, player, pos, velocity, kind, lifetime,
+            mapBlock):
         Unit.__init__(self)
 
+        self.world = world
         self.id = id
         self.team = team
         self.pos = tuple(pos)
-        self.angle = angle
         self.originatingPlayer = player
-        self.vel = (self.speed * sin(angle),
-                    -self.speed * cos(angle))
-        self.timeLeft = self.lifetime
-        self.turretShot = turret
-        self.ricochetShot = ricochet
+        self.vel = tuple(velocity)
+        self.timeLeft = lifetime
+        self.kind = kind
+        self.currentMapBlock = mapBlock
 
-        self.originatingPlayer.addShot(self)
-        
-        i, j = MapLayout.getMapBlockIndices(self.pos[0], self.pos[1])
-        try:
-            self.currentMapBlock = self.originatingPlayer.universe.zoneBlocks[i][j]
-            self.originatingPlayer.universe.zoneBlocks[i][j].addShot(self)
-        except IndexError:
-            logging.writeException()
-            print "Init: Shot off map"
-            self.kill()
-            return
         checkpoint('Shot created')
 
     def solid(self):
-        return not self.turretShot
+        return self.kind != Shot.TURRET
     
-    def movedBy(self, deltaX, deltaY):
-        # Called by moveUnit().
-        pass
-
     def update(self, deltaT):
         '''Called by the universe when this shot should update its position.
         deltaT is the time that's passed since its state was current, measured
@@ -184,7 +167,7 @@ class Shot(Unit):
         # Shots have a finite lifetime.
         self.timeLeft = self.timeLeft - deltaT
         if self.timeLeft <= 0:
-            self.kill()
+            self.world.shotExpired(self)
             return
 
         # Remember where the shot was - this is so that collisions with path
@@ -193,11 +176,11 @@ class Shot(Unit):
         
         delta = (self.vel[0]*deltaT,
                  self.vel[1]*deltaT)
-        try:
+        if True: #try:
             obstacle = self.currentMapBlock.moveUnit(self, *delta)
             if obstacle is not None:
                 # Shot hit an obstacle.
-                if self.ricochetShot:
+                if self.kind == Shot.RICOCHET:
                     obsAngle = atan2(obstacle.deltaPt[1], obstacle.deltaPt[0])
                     shotAngle = atan2(delta[1], delta[0])
                     dif=shotAngle -obsAngle
@@ -205,13 +188,10 @@ class Shot(Unit):
                     speed = sqrt(self.vel[0]*self.vel[0] + self.vel[1]*self.vel[1])
                     self.vel = (speed*cos(final), speed*sin(final))
                 else:
-                    self.kill()
+                    self.world.shotExpired(self)
                     return
-        except AttributeError:
-            print "Update: Shot off map"
-            print MapLayout.getMapBlockIndices(self.pos[0], self.pos[1])
-            self.kill()
+#        except AttributeError:
+#            print "Update: Shot off map"
+#            print MapLayout.getMapBlockIndices(self.pos[0], self.pos[1])
+#            self.world.shotExpired(self)
 
-    def kill(self):
-        self.originatingPlayer.destroyShot(self.id)
-        self.originatingPlayer.universe.removeShot(self)
