@@ -15,25 +15,19 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-import pickle
 import os
 import base64
-import time
 
 from twisted.internet import task
 
-from trosnoth.src.network.setup import ReadyForDynamicDetailsMsg
-from trosnoth.src import serverUniverse, serverLayout
-from trosnoth.src.model.universe_base import GameState
 from trosnoth.src.utils.unrepr import unrepr
 from trosnoth.src.utils.utils import timeNow
 from trosnoth.src.utils import logging
 
 from trosnoth.src.utils.jsonImport import json
 from trosnoth.src.utils.components import Component, handler, Plug
-from trosnoth.src.network.networkServer import serverMsgs
-from trosnoth.src.network.networkClient import Client, clientMsgs
-from trosnoth.src.messages.connection import *
+from trosnoth.src.network.server import serverMsgs
+from trosnoth.src.network.client import clientMsgs
 
 from trosnoth.data import getPath, user, makeDirs
 
@@ -42,6 +36,7 @@ def makeClient(app, filename):
     Sets up a replay client and game and connects them together.
     Returns the Client object.
     '''
+    assert False
     server = ReplayGame(filename)
     client = Client(app)
 
@@ -116,6 +111,7 @@ class ReplayGame(Component):
 
 
     def start(self):
+        assert False
         self.plug.send(GotSettings(self.getClientSettings()))
 
         self.startTime = timeNow()
