@@ -15,33 +15,31 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-from twisted.internet import reactor
-import pygame
+import random
 
 from trosnoth.src.gui.framework import framework
 from trosnoth.src.gui.common import Location, FullScreenAttachedPoint, ScaledSize, Area
-from trosnoth.src.gui.framework.unobtrusiveValueGetter import NumberGetter, YesNoGetter
+from trosnoth.src.gui.framework.unobtrusiveValueGetter import YesNoGetter
 
 from trosnoth.src.trosnothgui.ingame.messagebank import MessageBank
 from trosnoth.src.trosnothgui.ingame.stars import StarGroup
 from trosnoth.src.trosnothgui.ingame.udpnotify import UDPNotificationBar
 from trosnoth.src.trosnothgui.ingame import mainMenu
-from trosnoth.src.trosnothgui.ingame.transactionGUI import TransactionGUI
 from trosnoth.src.trosnothgui.ingame.gauges import TurretGauge, RespawnGauge, UpgradeGauge
 from trosnoth.src.trosnothgui.pregame.settingsMenu import SettingsMenu
 
 from trosnoth.src.base import MenuError
-import trosnoth.src.model.transaction as transactionModule
-from trosnoth.src.model.upgrades import Turret, Shield, MinimapDisruption, PhaseShift, Grenade, Ricochet, upgradeNames
+from trosnoth.src.model.upgrades import Turret, Shield, MinimapDisruption, PhaseShift, Grenade, Ricochet
 from trosnoth.src.model.team import Team
 from trosnoth.src.model.player import Player
 
-from trosnoth.src.network.transactions import *
-from trosnoth.src.messages.game import *
+from trosnoth.src.messages import (
+        BuyUpgradeMsg, DeleteUpgradeMsg, SetCaptainMsg,
+        TeamIsReadyMsg, RespawnRequestMsg)
 
 class DetailsInterface(framework.CompoundElement):
     '''Interface containing all the overlays onto the screen:
-    chat messages, player lists, gauges, stars, transaction requests, etc.'''
+    chat messages, player lists, gauges, stars, etc.'''
     def __init__(self, app, gameInterface):
         super(DetailsInterface, self).__init__(app)
         world = gameInterface.world
@@ -51,7 +49,6 @@ class DetailsInterface(framework.CompoundElement):
         maxView = 8
 
         self.world = world
-        rect = pygame.Rect((0,0), self.app.screenManager.scaledSize)
         self.player = None
         self.currentMessages = MessageBank(self.app, maxView, 50,
                                            Location(FullScreenAttachedPoint(ScaledSize(-40,-40),'bottomright'), 'bottomright'),
@@ -60,8 +57,6 @@ class DetailsInterface(framework.CompoundElement):
         self.currentChats = MessageBank(self.app, maxView, 50, Location(FullScreenAttachedPoint(ScaledSize(40,256),'topleft'), 'topleft'), 'left', 'top',
                                         self.app.screenManager.fonts.messageFont)
         # If we want to keep a record of all messages and their senders
-        self.allMessages = []
-        self.transactionGUI = None
         self.input = None
         self.inputText = None
         self.unobtrusiveGetter = None
@@ -81,6 +76,54 @@ class DetailsInterface(framework.CompoundElement):
     def controller(self):
         return self.gameInterface.controller
 
+    def tick(self, deltaT):
+        super(DetailsInterface, self).tick(deltaT)
+        player = self.player
+        if player is None:
+            return
+        self._updateTurretGauge()
+        self._updateRespawnGauge()
+        self._updateUpgradeGauge()
+
+    def _updateTurretGauge(self):
+        player = self.player
+        if self.turretGauge is None:
+            if player.turret:
+                self.turretGauge = TurretGauge(self.app, Area(
+                        FullScreenAttachedPoint(ScaledSize(0,-60),
+                        'midbottom'), ScaledSize(100,30), 'midbottom'),
+                        player)
+                self.elements.append(self.turretGauge)
+        elif not player.turret:
+            self.elements.remove(self.turretGauge)
+            self.turretGauge = None
+            
+    def _updateRespawnGauge(self):
+        player = self.player
+        if self.respawnGauge is None:
+            if player.ghost:
+                self.respawnGauge = RespawnGauge(self.app, Area(
+                        FullScreenAttachedPoint(ScaledSize(0,-20),
+                        'midbottom'), ScaledSize(100,30), 'midbottom'),
+                        player)
+                self.elements.append(self.respawnGauge)
+        elif not player.ghost:
+            self.elements.remove(self.respawnGauge)
+            self.respawnGauge = None
+            
+    def _updateUpgradeGauge(self):
+        player = self.player
+        if self.upgradeGauge is None:
+            if player.upgrade is not None:
+                self.upgradeGauge = UpgradeGauge(self.app, Area(
+                        FullScreenAttachedPoint(ScaledSize(0,-20),
+                        'midbottom'), ScaledSize(100,30), 'midbottom'),
+                        player)
+                self.elements.append(self.upgradeGauge)
+        elif player.upgrade is None:
+            self.elements.remove(self.upgradeGauge)
+            self.upgradeGauge = None
+
     def gameOver(self, winningTeam):
         self.gameInterface.gameOver(winningTeam)
 
@@ -98,32 +141,9 @@ class DetailsInterface(framework.CompoundElement):
             self.elements.remove(self.settingsMenu)
             self.gameInterface.updateKeyMapping()
 
-    def _getUpgrade(self, stars, upgrade):
-        try:
-            startingStars = int(stars)
-        except:
-            raise MenuError, "Please enter a valid value"
-        else:
-            if not self.player.hasThisManyStars(startingStars):
-                raise MenuError, "You do not have that many stars"
-            else:
-                self.controller.send(StartingTransactionMsg(self.player.team.id, self.player.id, stars, upgrade.upgradeType))
-
     def _requestUpgrade(self, upgradeType):
-        result, reason = self.player.canRequestUpgrade(upgradeType)
-        if result:
-            self.getStars('How many stars to contribute from the start?', self._getUpgrade, upgradeType)
-        else:
-            raise MenuError, reason
-
-    def _addStars(self, numStars):
-        result, reason = self.player.canAddStarsNow(numStars)
-        if result:
-            self.controller.send(AddingStarsMsg(self.player.team.id, self.player.id, numStars))
-        else:
-            raise MenuError, reason
-        
-        
+        self.controller.send(BuyUpgradeMsg(self.player.id,
+                upgradeType.upgradeType, random.randrange(1<<32)))
 
     def doAction(self, action):
         '''
@@ -152,17 +172,15 @@ class DetailsInterface(framework.CompoundElement):
             if self.gameInterface.world.replay:
                 # Replay-specific: follow the action.
                 self.gameInterface.gameViewer.setTarget(None)
-    
+        elif action == 'toggle terminal':
+            self.gameInterface.toggleTerminal()
+
         # All actions after this line should require a player.
         if self.player is None:
             return
         if action == 'respawn':
-            result, reason = self.player.canRespawn()
-            if result:
-                self.player.requestRespawn()
-            else:
-                self.newMessage(reason,
-                        self.app.theme.colours.errorMessageColour)
+            self.controller.send(RespawnRequestMsg(self.player.id,
+                    random.randrange(1<<32)))
         elif action == 'turret':
             self._requestUpgrade(Turret)
             self.menuManager.manager.reset()
@@ -180,23 +198,6 @@ class DetailsInterface(framework.CompoundElement):
             self.menuManager.manager.reset()
         elif action == 'ricochet':
             self._requestUpgrade(Ricochet)
-            self.menuManager.manager.reset()
-        elif action == 'use upgrade':
-            result, reason = self.player.canUseUpgrade()
-            if result:
-                self.player.requestUse()
-            else:
-                self.newMessage(reason,
-                        self.app.theme.colours.errorMessageColour)
-            self.menuManager.manager.reset()
-        elif action == 'add stars':
-            result, reason = self.player.canAddStars()
-            if result:
-                self.getStars('How many stars to contribute?', self._addStars)
-            else:
-                self.newMessage(reason,
-                        self.app.theme.colours.errorMessageColour)
-            
             self.menuManager.manager.reset()
         elif action == 'abandon':
             self.abandon(self.player)
@@ -214,97 +215,30 @@ class DetailsInterface(framework.CompoundElement):
             if self.menuManager is not None:
                 self.menuManager.showBuyMenu()
 
-    def transactionStarted(self, transaction):
-        '''Called when a transaction is started, so it can display this
-        information to the user'''
-        # There may be an existing transaction which is completed/abandoned, but still on screen
-        if self.transactionGUI is not None:
-            if self.transactionGUI.transaction.state not in (transactionModule.TransactionState.Abandoned, transactionModule.TransactionState.Completed):
-                # We must have had some bad information
-                print 'We had incorrect transaction information.'
-            self._deleteTransGUI(self.transactionGUI)
-
-        # If we're not focusing on a player, do not show the GUI.
-        # Although perhaps if we were 'focusing' on a team, we could...
-        # Count that as a TODO.
-        if self.player is None or transaction.purchasingTeam is not self.player.team:
-            return
-
-
-        self.transactionGUI = TransactionGUI(self.app, Location(FullScreenAttachedPoint(ScaledSize(200,0), 'topleft'), 'topleft'), transaction, self.player)
-        self.elements.append(self.transactionGUI)
-
-        self.newMessage('%s purchasing %s' % (
-                transaction.purchasingPlayer.nick, upgradeNames[
-                transaction.upgrade]),
-                self.app.theme.colours.transactionMessageColour)
-
-    # Set up a message to display to the screen when someone adds star(s)
-    def addStars(self, player, stars):
-        if player.team == self.player.team:
-            self.newMessage('%s added %d star%s' % (player.nick, stars,
-                    ('s', '')[stars == 1]),
-                    self.app.theme.colours.transactionMessageColour)
-        else:
-            print 'INEFFICIENCY: being sent enemy\'s star additions'
-
-    def _deleteTransGUI(self, transactionGUI):
-        '''Removes the TransactionGUI object from the screen'''
-        # This check is necessary in case another one was created
-        # while one was still up
-        if transactionGUI is self.transactionGUI:
-            if transactionGUI in self.elements:
-                self.elements.remove(transactionGUI)
-            else:
-                print 'SOMEWHAT BAD: transactionGUI not in elements'
-            self.transactionGUI = None
-        else:
-            print 'WARNING: Wrong transactionGUI being deleted'
-            
-    def transactionChanged(self, transaction):
-        if self.transactionGUI is None or self.transactionGUI.transaction != transaction:
-            self.transactionStarted(transaction)
-
-    def transactionAbandoned(self, transaction):
-        self.newMessage("%s's Transaction Abandoned" % (repr(transaction.purchasingTeam)),
-                self.app.theme.colours.transactionMessageColour)
-        if self.transactionGUI is not None and self.transactionGUI.transaction == transaction:
-            reactor.callLater(3, self._deleteTransGUI, self.transactionGUI)
-
-    def transactionComplete(self, transaction):
-        self.newMessage("%s's Transaction Completed" % (repr(transaction.purchasingTeam)),
-                self.app.theme.colours.transactionMessageColour)
-        if self.transactionGUI is not None and self.transactionGUI.transaction == transaction:
-            reactor.callLater(3, self._deleteTransGUI, self.transactionGUI)
-
     def newMessage(self, text, colour=None):
         if colour is None:
             colour = self.app.theme.colours.grey
         self.currentMessages.newMessage(text, colour)
-        self.allMessages.append((text))
 
     def newChat(self, text, sender, private):
         if sender is self.player:
             # Don't bother receiving messages from myself.
             return
-        
+
         nick = sender.nick
         if not private:
             # Message for everyone
             extra = ': '
-            self.allMessages.append((nick, text))
-            
+
         elif isinstance(private, Player) and self.player is private:
             # Destined for the one player
             extra = ' (private): '
-            self.allMessages.append((nick, '#P# ' + text))
-            
+
         elif isinstance(private, Team) and self.player is not None and \
                 self.player.team == private:
             # Destined for the one team.
             extra = ': '
-            self.allMessages.append((nick, text))
-            
+
         else:
             # May not have been destined for our player after all.
             print "Don't want this text"
@@ -313,28 +247,16 @@ class DetailsInterface(framework.CompoundElement):
         self.currentChats.newMessage(allText,
                 self.app.theme.colours.chatColour(sender.team))
 
-    def sentChat(self, text, team, private):
-        if not private:
-            # Message for everyone
-            extra = 'Me (open): '
-            self.allMessages.append(('Me (open):', text))
-            
-        elif isinstance(private, Player):
-            # Destined for the one player
-            extra = 'Me (to_%s): ' % (private.nick)
-            self.allMessages.append((extra, '#P# ' + text))
-            
-        elif isinstance(private, Team):
-            # Destined for the one team.
-            extra = 'Me: '
-            self.allMessages.append(('Me', text))
-            
-        else:
-            # May not have been destined for our player after all.
-            print "Don't want this text"
-            return
-        allText = extra + text
-        self.currentChats.newMessage(allText,
+    def sentOpenChat(self, team, text):
+        self.currentChats.newMessage('Me (open): %s' % (text,),
+                self.app.theme.colours.chatColour(team))
+
+    def sentPrivateChat(self, team, nick, text):
+        self.currentChats.newMessage('Me (to_%s): %s' % (nick, text),
+                self.app.theme.colours.chatColour(team))
+
+    def sentPublicChat(self, team, text):
+        self.currentChats.newMessage('Me: %s' % (text,),
                 self.app.theme.colours.chatColour(team))
 
     def processEvent(self, event):
@@ -368,7 +290,7 @@ class DetailsInterface(framework.CompoundElement):
         if self.menuManager is not None and self.menuManager not in self.elements:
             self.elements.append(self.menuManager)
         self.input = self.inputText = None
-        
+
 
     def inputStarted(self):
         self.elements.append(self.input)
@@ -388,40 +310,19 @@ class DetailsInterface(framework.CompoundElement):
             return
 
         self.gameInterface.gameViewer.openChat(self.player)
-        
-
-    def getStars(self, message, function, *params):
-        '''Called when a number of stars is requested.
-        function is the function that should be called when the number of
-        stars has been inputted'''
-        self.endInput()
-        self.unobtrusiveGetter = NumberGetter(self.app, Location(
-                FullScreenAttachedPoint((0,0), 'center'), 'center'), message,
-                self.app.screenManager.fonts.unobtrusivePromptFont,
-                self.app.theme.colours.unobtrusivePromptColour, 3)
-        self.elements.append(self.unobtrusiveGetter)
-        def gotValue(num):
-            if self.unobtrusiveGetter:
-                self.elements.remove(self.unobtrusiveGetter)
-                self.unobtrusiveGetter = None
-            function(num, *params)
-                
-        self.unobtrusiveGetter.onGotValue.addListener(gotValue)
 
 
     def abandon(self, player):
-        '''Called when a player says they wish to abandon their upgrade, or
-        the transaction they initiated'''
+        '''
+        Called when a player says they wish to abandon their upgrade.
+        '''
         if player.upgrade:
-            addOn = 'Upgrade'
+            addOn = 'upgrade'
             if player.upgrade.inUse and type(player.upgrade) == Grenade:
                 self.newMessage('Cannot abandon an active grenade!',
                         self.app.theme.colours.errorMessageColour)
                 raise MenuError, "Cannot abandon an active grenade!"
-                
-        elif player.team.currentTransaction and \
-                player.team.currentTransaction.purchasingPlayer is player:
-            addOn = 'Transaction'
+
         else:
             return
 
@@ -438,64 +339,35 @@ class DetailsInterface(framework.CompoundElement):
                 self.elements.remove(self.unobtrusiveGetter)
                 self.unobtrusiveGetter = None
             if abandon:
-                player.abandon()
-                
+                self.controller.send(DeleteUpgradeMsg(player.id))
+
         self.unobtrusiveGetter.onGotValue.addListener(gotValue)
-        
 
 
     def upgradeUsed(self, player):
-        message = '%s is using %s' % (player.nick, \
-                                      repr(player.upgrade))
+        upgrade = player.upgrade
+        if upgrade is None:
+            upgrade = 'an upgrade'
+        else:
+            upgrade = repr(upgrade)
+        message = '%s is using %s' % (player.nick, player.upgrade)
         self.newMessage(message)
-        
-        if player is self.player:
-            if self.upgradeGauge is not None:
-                # Remove the existing one.
-                print 'Stray upgradeGauge!'
-                self.elements.remove(self.upgradeGauge)
-            self.upgradeGauge = UpgradeGauge(self.app, Area(FullScreenAttachedPoint(ScaledSize(0,-20), 'midbottom'), ScaledSize(100,30), 'midbottom'), player)
-            self.elements.append(self.upgradeGauge)
-
 
     def upgradeDestroyed(self, player, upgrade):
-        message = "%s's %s is gone" % (player.nick, \
-                                       repr(upgrade))
+        if upgrade is None:
+            upgrade = 'upgrade'
+        else:
+            upgrade = repr(upgrade)
+        message = "%s's %s is gone" % (player.nick, upgrade)
         self.newMessage(message)
-        
-        if player is self.player and self.upgradeGauge is not None:
-            self.elements.remove(self.upgradeGauge)
-            self.upgradeGauge = None
-
-    def turretStarted(self, player):
-        if player is self.player:
-            self.turretGauge = TurretGauge(self.app, Area(FullScreenAttachedPoint(ScaledSize(0,-60), 'midbottom'), ScaledSize(100,30), 'midbottom'), player)
-            self.elements.append(self.turretGauge)
-
-    def turretEnded(self, player):
-        if player is self.player:
-            self.elements.remove(self.turretGauge)
-            self.turretGauge = None
-
-    def playerKilled(self, player, killer):
-        if player is self.player:
-            if self.respawnGauge is not None:
-                self.elements.remove(self.respawnGauge)
-            self.respawnGauge = RespawnGauge(self.app, Area(FullScreenAttachedPoint(ScaledSize(0,-20), 'midbottom'), ScaledSize(100,30), 'midbottom'), player)
-            self.elements.append(self.respawnGauge)
-
-    def playerRespawned(self, player):
-        if player is self.player and self.respawnGauge is not None:
-            self.elements.remove(self.respawnGauge)
-            self.respawnGauge = None
 
     def becomeCaptain(self):
         if self.player:
-            self.controller.send(SetCaptain(self.player))
+            self.controller.send(SetCaptainMsg(self.player.id))
 
     def teamReady(self):
         if self.player:
-            self.controller.send(TeamIsReady(self.player))
+            self.controller.send(TeamIsReadyMsg(self.player.id))
 
     def showPlayerDetails(self):
         self.gameInterface.gameViewer.toggleLeaderBoard()
