@@ -18,7 +18,6 @@
 from twisted.internet import reactor
 
 from trosnoth.src.gui.framework import framework, elements
-from trosnoth.src.trosnothgui.ingame import colours
 from trosnoth.src.gui import colours as guiColours
 
 from trosnoth.src.gui.common import Location, RelativeLocation
@@ -145,7 +144,7 @@ class Message(elements.TextElement):
     def __init__(self, app, text, colour, messageBank, timeLimit):
         self.messageBank = messageBank
         if not colour:
-            colour = colours.grey
+            colour = app.theme.colours.grey
         super(Message, self).__init__(app, text, self.messageBank.font,
                                       self.messageBank.startPt,
                                       colour, shadow = True)
