@@ -23,20 +23,20 @@ import pygame
 
 
 class ReplayMenu(InGameMenu):
-    def __init__(self, app, client, world):
+    def __init__(self, app, world, onDisconnectRequest=None):
         super(ReplayMenu, self).__init__(app)
         self.elements = []
-        self.client = client
 
         item = elements.TextButton(app, ScaledLocation(0, 768, 'bottomleft'),
                 'End Replay', self.font,
                 app.theme.colours.inGameButtonColour,
                 app.theme.colours.white)
         item.onClick.addListener(self.closeReplay)
-        self.elements.append(item)
 
-    def closeReplay(self, sender):
-        self.client.disconnect()
+        self.onDisconnectRequest = Event()
+        if onDisconnectRequest is not None:
+            self.onDisconnectRequest.addListener(onDisconnectRequest)
+        self.elements.append(item)
 
 
 class ViewControlInterface(framework.Element):
