@@ -22,9 +22,8 @@ from math import sin, cos
 
 import pygame
 
-from trosnoth.src.utils.utils import new, timeNow
+from trosnoth.src.utils.utils import timeNow
 from trosnoth.src.utils.logging import writeException
-from trosnoth.src.model import universe
 from trosnoth.src.trosnothgui.ingame.minimap import MiniMap
 from trosnoth.src.gui.framework import framework, elements
 from trosnoth.src.trosnothgui.ingame.leaderboard import LeaderBoard
@@ -37,7 +36,7 @@ from trosnoth.src.model.universe_base import GameState
 from trosnoth.src.model.map import MapLayout
 from trosnoth.src.trosnothgui.ingame.sprites import PlayerSprite
 
-from trosnoth.src.gui.common import Location, FullScreenAttachedPoint, ScaledSize
+from trosnoth.src.gui.common import Location, FullScreenAttachedPoint
 
 class ViewManager(framework.CompoundElement):
     '''A ViewManager object takes a given universe, and displays a screenfull
@@ -72,8 +71,6 @@ class ViewManager(framework.CompoundElement):
         self.setTarget(target)
         self.universe = universe
         
-        self.sRect = pygame.Rect((0,0), self.app.screenManager.size)
-        
         # self._focus represents the point where the viewManager is currently
         # looking.
         self._focus = (universe.map.layout.centreX, universe.map.layout.centreY)
@@ -81,11 +78,12 @@ class ViewManager(framework.CompoundElement):
         self.speed = 0          # Speed that the focus is moving.
         
         # Create a surface for storing the background stuff.
-        self.backdrop = pygame.surface.Surface(self.sRect.size).convert()
-        self.spareCanvas = pygame.surface.Surface(self.sRect.size).convert()
-        
+        self.sRect = None
+        self.backdrop = None
+        self.spareCanvas = None
+
         # Now fill the backdrop with what we're looking at now.
-        self.drawBackdropRect(self.sRect)
+        self.appResized()
 
     def setTarget(self, target):
         '''Makes the viewManager's target the specified value.'''
@@ -111,12 +109,19 @@ class ViewManager(framework.CompoundElement):
         '''
         Called when the app has been resized.
         '''
-        self.sRect = pygame.Rect((0,0), self.app.screenManager.size)
+        self.sRect = sRect = pygame.Rect((0,0), self.app.screenManager.size)
+        centre = sRect.center
+        sRect.width = min(1024, sRect.width)
+        sRect.height = min(768, sRect.height)
+        sRect.center = centre
+
         # Create a surface for storing the background stuff.
-        self.backdrop = pygame.surface.Surface(self.sRect.size).convert()
-        self.spareCanvas = pygame.surface.Surface(self.sRect.size).convert()
+        self.backdrop = pygame.surface.Surface(sRect.size).convert()
+        self.spareCanvas = pygame.surface.Surface(sRect.size).convert()
+
         # Now fill the backdrop with what we're looking at now.
-        self.drawBackdropRect(self.sRect)
+        r = pygame.Rect((0, 0), sRect.size)
+        self.drawBackdropRect(r)
 
     def draw(self, screen):
         '''Draws the current state of the universe at the current viewing
@@ -149,12 +154,14 @@ class ViewManager(framework.CompoundElement):
                                               -x, yHeight))
         
         # Now put the backdrop onto the screen.
-        screen.blit(self.backdrop, (0,0))
+        if self.sRect.topleft != (0, 0):
+            screen.fill((0, 0, 0))
+        screen.blit(self.backdrop, self.sRect)
         
         # Go through and update the positions of the players on the screen.
         ntGroup = pygame.sprite.Group()
         visPlayers = pygame.sprite.Group()
-        for player in self.universe.players:
+        for player in self.universe.iterPlayers():
             # Calculate the position of the player.
             player.rect.center = [player.pos[i] - self._focus[i] + \
                     self.sRect.center[i] for i in (0,1)]
@@ -176,6 +183,7 @@ class ViewManager(framework.CompoundElement):
                 
                 # Place the star rectangle below the nametag.
                 mx, my = player.nametag.rect.midbottom
+                player.starTally.setStars(player.stars)
                 player.starTally.rect.midtop = (mx, my-5)
                 ntGroup.add(player.starTally)
         
@@ -185,19 +193,20 @@ class ViewManager(framework.CompoundElement):
         ntGroup.empty()
         
         # Draw the shots.
-        for shot in self.universe.shots:
-            # Calculate the position of the player.
+        for shot in self.universe.iterShots():
+            # Calculate the position of the shot.
             shot.rect.center = [shot.pos[i] - self._focus[i] + \
                     self.sRect.center[i] for i in (0,1)]
-        
-        self.universe.shots.draw(screen)
+            if shot.rect.colliderect(self.sRect):
+                screen.blit(shot.image, shot.rect)
         try:
             # Draw the grenades.
-            for grenade in self.universe.grenades:
+            for grenade in self.universe.iterGrenades():
                 # Calculate the position of the player.
                 grenade.rect.center = [grenade.pos[i] - self._focus[i] + \
                         self.sRect.center[i] for i in (0,1)]
-            self.universe.grenades.draw(screen)
+                if grenade.rect.colliderect(self.sRect):
+                    screen.blit(grenade.image, grenade.rect)
         except:
             writeException()
 
@@ -207,8 +216,12 @@ class ViewManager(framework.CompoundElement):
         self.backdrop.fill((64, 0, 64), rect)
         
         # Find which map blocks are within the bounds of that rect.
-        i, j = MapLayout.getMapBlockIndices(self._focus[0] - self.sRect.center[0] + rect.left - self.sRect.left,
-                                                     self._focus[1] - self.sRect.center[1] + rect.top - self.sRect.top)
+        i, j = MapLayout.getMapBlockIndices(self._focus[0] -
+                self.sRect.center[0] + rect.left + self.sRect.left,
+                self._focus[1] - self.sRect.center[1] + rect.top +
+                self.sRect.top)
+
+        blockRect = pygame.Rect(0, 0, 0, 0)
         
         y = i
         while True:
@@ -231,12 +244,18 @@ class ViewManager(framework.CompoundElement):
                     break
                 
                 # Put the block in its rightful place.
-                block.drawRect.topleft = [block.pos[n] - self._focus[n] + \
-                                          self.sRect.center[n] for n in (0,1)]
+                blockRect.size = block.defn.rect.size
+                blockRect.topleft = (
+                    block.defn.pos[0] - self._focus[0] + self.sRect.centerx -
+                        self.sRect.left,
+                    block.defn.pos[1] - self._focus[1] + self.sRect.centery -
+                        self.sRect.top
+                )
+
                 # Draw the block.
-                if block.drawRect.colliderect(rect):
-                    crop = rect.clip(block.drawRect)
-                    block.draw(self.backdrop, crop)
+                if blockRect.colliderect(rect):
+                    crop = rect.clip(blockRect)
+                    self._drawBlock(block, blockRect, crop)
                     x += 1
                     first = False
                 else:
@@ -246,6 +265,85 @@ class ViewManager(framework.CompoundElement):
                 # Not even the first one was drawn; it must have drawn enough
                 # rows
                 break
+
+    def _drawBlock(self, block, drawRect, area):
+        if block.defn.kind in ('top', 'btm'):
+            self._drawBodyBlock(block, drawRect, area)
+        else:
+            self._drawInterfaceBlock(block, drawRect, area)
+
+    def _drawBlockArtwork(self, block, drawRect, area):
+        if block.defn.graphics is None:
+            return
+
+        # Find its rightful place
+        rp = (area.left - drawRect.left, area.top - drawRect.top)
+        rr = pygame.rect.Rect(rp, area.size)
+        if self.app.displaySettings.showObstacles:
+            self.backdrop.blit(block.defn.debugGraphics.graphic, area.topleft,
+                    rr)
+        else:
+            self.backdrop.blit(block.defn.graphics.graphic, area.topleft, rr)
+
+    def _drawBodyBlock(self, block, drawRect, area):
+        if block.zone:
+            # First draw the background the colour of the zone.
+            clr = self._getZoneBackgroundColour(block.zone)
+            self.backdrop.fill(clr, area)
+
+            self._drawBlockArtwork(block, drawRect, area)
+            
+            # Draw the orb.
+            img = self._getOrbImage(block.zone)
+            r = img.get_rect()
+            r.center = block.orbPos(drawRect)
+            self.backdrop.blit(img, r)
+        else:
+            self._drawBlockArtwork(block, drawRect, area)
+
+    def _getOrbImage(self, zone):
+        if zone.orbOwner is not None:
+            return self.app.theme.sprites.orb(zone.orbOwner)
+        else:
+            return self.app.theme.sprites.neutralOrb
+
+    def _drawInterfaceBlock(self, block, drawRect, area):
+        # First draw the background the colour of the zone.
+        if not (block.zone1 or block.zone2):
+            # No zones to draw
+            return
+        
+        if (block.zone1 and block.zone2 and block.zone1.zoneOwner == 
+                block.zone2.zoneOwner):
+            # Same colour; simply fill in that colour
+            clr = self._getZoneBackgroundColour(block.zone1)
+            self.backdrop.fill(clr, area)
+        else:
+            if not block.zone1:
+                z1 = None
+            else:
+                if block.zone1.zoneOwner:
+                    z1 = block.zone1.zoneOwner.id
+                else:
+                    z1 = '\x00'
+            if not block.zone2:
+                z2 = None
+            else:
+                if block.zone2.zoneOwner:
+                    z2 = block.zone2.zoneOwner.id
+                else:
+                    z2 = '\x00'
+                    
+            # This is getting a pre-made image for the background colour
+            img = self.app.theme.sprites.interfaceBackground(block.defn.kind,
+                    z1, z2)
+
+            rp = (area.left - drawRect.left, area.top - drawRect.top)
+            rr = pygame.rect.Rect(rp, area.size)
+            self.backdrop.blit(img, area.topleft, rr)
+
+        # Now draw the obstacles.
+        self._drawBlockArtwork(block, drawRect, area)
                                                
     def updateFocus(self):
         '''Updates the location that the ViewManager is focused on.  First
@@ -274,7 +372,7 @@ class ViewManager(framework.CompoundElement):
                             plPos[1] - radius * cos(plAngle))
 
             # If the player no longer exists, look wherever we want.
-            if self.target not in self.universe.players:
+            if not self.universe.hasPlayer(self.target):
                 self.setTarget(None)
         elif isinstance(self.target, (tuple, list)):
             targetPt = self.target
@@ -284,10 +382,10 @@ class ViewManager(framework.CompoundElement):
 
             # First check for non-existent players.
             for p in players:
-                if p not in self.universe.players:
+                if not self.universe.hasPlayer(p):
                     players.remove(p)
 
-            if len(self.universe.players) == 0:
+            if self.universe.getPlayerCount() == 0:
                 # No players anywhere. No change.
                 targetPt = tuple(self._focus)
             elif len(players) == 0:
@@ -296,12 +394,11 @@ class ViewManager(framework.CompoundElement):
                 maxP = 0
                 curZone = None
                 for z in self.universe.zones:
-                    count = len(z.players) + len(z.nonPlayers)
+                    count = len(self.universe.getPlayersInZone(z))
                     if count > maxP:
                         maxP = count
                         curZone = z
-                self.autoFocusInfo[1] = [self.universe.getPlayerSprite(p) for
-                    p in (list(curZone.players) + list(curZone.nonPlayers))]
+                self.autoFocusInfo[1] = self.universe.getPlayersInZone(curZone)
 
                 targetPt = tuple(self._focus)
                 countdown = 10
@@ -315,7 +412,7 @@ class ViewManager(framework.CompoundElement):
                 if countdown <= 0:
                     # Count players in view.
                     numInView = 0
-                    for p in self.universe.players:
+                    for p in self.universe.iterPlayers():
                         if abs(p.pos[0] - targetPt[0]) < self.app.screenManager.scaledSize[0] and \
                                 abs(p.pos[1] - targetPt[1]) < self.app.screenManager.scaledSize[1]:
                             numInView += 1
@@ -325,16 +422,11 @@ class ViewManager(framework.CompoundElement):
                         maxP = 0
                         curZone = None
                         for z in self.universe.zones:
-                            count = len(z.players) + len(z.nonPlayers)
+                            count = len(self.universe.getPlayersInZone(z))
                             if count > maxP:
                                 maxP = count
                                 curZone = z
-                        self.autoFocusInfo[1] = list(curZone.players) + \
-                                                list(curZone.nonPlayers)
-                        self.autoFocusInfo[1] = [
-                                self.universe.getPlayerSprite(p) for p in
-                                (list(curZone.players) +
-                                list(curZone.nonPlayers))]
+                        self.autoFocusInfo[1] = self.universe.getPlayersInZone(z)
                     countdown = 10
                 countdown -= 1
         
@@ -377,15 +469,23 @@ class ViewManager(framework.CompoundElement):
         return deltaBackdrop
 
     def reDraw(self):
-        self.drawBackdropRect(self.sRect)
+        r = pygame.Rect((0, 0), self.sRect.size)
+        self.drawBackdropRect(r)
+
+    def _getZoneBackgroundColour(self, zone):
+        colours = self.app.theme.colours
+        if zone.zoneOwner:
+            return colours.backgroundColour(zone.zoneOwner)
+        else:
+            return colours.neutral_bg
 
 class PregameMessage(elements.TextElement):
-    def __init__(self, app, pos = None):
+    def __init__(self, app, pos=None):
         if pos is None:
             pos = Location(FullScreenAttachedPoint((0, -150), 'center'), 'center')
         super(PregameMessage, self).__init__(
             app,
-            'Game will begin when both captains signal ready.',
+            '',
             app.screenManager.fonts.bigMenuFont,
             pos,
             app.theme.colours.pregameMessageColour,
@@ -397,8 +497,6 @@ class GameViewer(framework.CompoundElement):
     '''The gameviewer comprises a viewmanager and a minimap, which can be
     switched on or off.'''
 
-    # Needs to be declared here instead of in statusBar.py so that the
-    # leaderboard can use it
     zoneBarHeight = 25
 
     def __init__(self, app, gameInterface, world):
@@ -419,6 +517,7 @@ class GameViewer(framework.CompoundElement):
         self.makeMiniMap()
 
         self.elements = [self.viewManager, self.pregameMessage]
+        self._screenSize = tuple(app.screenManager.size)
 
         self.minimapDisruptor = None
         self.teamsDisrupted = set()
@@ -430,8 +529,10 @@ class GameViewer(framework.CompoundElement):
         '''
         Checks whether the application has resized and adjusts accordingly.
         '''
-        if self.viewManager.sRect.size == self.app.screenManager.size:
+        if self._screenSize == self.app.screenManager.size:
             return
+        self._screenSize = tuple(self.app.screenManager.size)
+
         self.viewManager.appResized()
         # Recreate the minimap.
         showHUD = self.miniMap in self.elements
@@ -444,21 +545,32 @@ class GameViewer(framework.CompoundElement):
             
 
     def makeMiniMap(self):
-        miniMapSize = (min(self.app.screenManager.scaledSize[0] / 100 * 35, 300),
-                    min(self.app.screenManager.scaledSize[1] / 100 * 35, 200))
-        self.miniMap = MiniMap(self.app, (self.app.screenManager.size[0] - miniMapSize[0] - 5, 5),
-                                       miniMapSize, 20, self.world,
-                                       self.viewManager)
+        self.miniMap = MiniMap(self.app, 20, self.world, self.viewManager)
         xPos = -10
-        yPos = self.miniMap.sRect.bottom + self.zoneBarHeight - 5
+        yPos = self.miniMap.getRect().bottom + self.zoneBarHeight - 5
         messagePos = Location(FullScreenAttachedPoint((xPos, yPos), 'topright'), 'topright')
         self.pregameMessage = PregameMessage(self.app, messagePos)
-        if self.world.gameState == GameState.InProgress:
-            self.pregameMessage.setText('')
         self.zoneBar = ZoneProgressBar(self.app, self.world, self)
         self.leaderboard = LeaderBoard(self.app, self.world, self)
 
         self.elements = [self.viewManager, self.pregameMessage]
+
+    def updatePregameMessage(self):
+        if self.world.gameState == GameState.InProgress:
+            text = ''
+        elif self.world.gameState == GameState.PreGame:
+            text = 'Game will begin when both captains signal ready.'
+        elif self.world.gameState == GameState.Starting:
+            text = 'Prepare yourself...'
+        else:
+            pi = self.interface.runningPlayerInterface
+            if pi is not None and pi.player is not None:
+                team = pi.player.team
+                if team is self.world.winningTeam:
+                    text = 'Congratulations!'
+                else:
+                    text = 'Better luck next time!'
+        self.pregameMessage.setText(text)
                 
     def setTarget(self, target):
         'Target should be a player, a point, or None.'
@@ -472,6 +584,7 @@ class GameViewer(framework.CompoundElement):
     def tick(self, deltaT):
         self.resizeIfNeeded()
         self.worldgui.tick(deltaT)
+        self.updatePregameMessage()
         super(GameViewer, self).tick(deltaT)
 
     def minimapDisruption(self, player):
