@@ -20,7 +20,8 @@ game to the screen, including menus and other doodads.'''
 
 import pygame
 
-from trosnoth.src.network import networkClient
+from trosnoth.src.network.client import ClientNetHandler
+from trosnoth.src.game import makeIsolatedGame
 from trosnoth.src import replays
 
 from trosnoth.src.gui.framework import framework, elements
@@ -30,20 +31,19 @@ from trosnoth.src.trosnothgui.pregame.connectingScreen import ConnectingScreen
 from trosnoth.src.trosnothgui.pregame.backdrop import TrosnothBackdrop
 
 from trosnoth.src.trosnothgui.pregame.connectionFailedDialog import ConnectionFailedDialog
-from trosnoth.src.trosnothgui.ingame.interfaceComponent import InterfaceComponent
 from trosnoth.src.trosnothgui.ingame.gameInterface import GameInterface
 
 class Interface(framework.CompoundElement):
     def __init__(self, app):
         super(Interface, self).__init__(app)
-        
+
         # Create an interfaces for pre- and post-connection.
         self.backdrop = TrosnothBackdrop(app)
         self.startupInterface = StartupInterface(app, self)
-        
+
         self._buildPointer()
         self.gi = None
-        
+
         self.elements = [self.backdrop, self.startupInterface]
 
     def _buildPointer(self):
@@ -54,11 +54,11 @@ class Interface(framework.CompoundElement):
 
     def processEvent(self, event):
         # Capture the quit event.
-        if event.type == pygame.QUIT:                
+        if event.type == pygame.QUIT:
             self.app.stop()
             return
         return super(Interface, self).processEvent(event)
-    
+
     def connectToServer(self, host, port):
         self.elements = [
             self.backdrop,
@@ -66,40 +66,51 @@ class Interface(framework.CompoundElement):
                 onCancel=self.cancelConnecting)
         ]
 
-        self.netClient = networkClient.NetworkClient(self.app)
-        self.netClient.onClosed.addListener(self.connectionLost)
-
-        result = self.netClient.connectByPort(host, port)
-        result.addCallback(self.connectionEstablished, self.netClient).addErrback(
+        self.netHandler = ClientNetHandler(self.app.netman)
+        result = self.netHandler.makeConnection(host, port)
+        result.addCallback(self.connectionEstablished).addErrback(
                 self.connectionFailed)
         return result
 
     def cancelConnecting(self, source):
-        self.netClient.cancelConnecting()
+        self.netHandler.cancelConnecting()
         self.startupInterface.mainMenu()
 
-    def connectionEstablished(self, world, netClient):
+    def connectionEstablished(self, settings):
         'Called when this client establishes a connection to a server.'
         self.startupInterface.mainMenu()
-        self.setupGame(world, netClient)
+
+        game, eventPlug, controlPlug = makeIsolatedGame(
+                self.app.layoutDatabase)
+        self.netHandler.requestPlug.connectPlug(controlPlug)
+        self.netHandler.eventPlug.connectPlug(eventPlug)
+
+        self.setupGame(game)
+        game.syncWorld()
 
         self.elements = [self.gi]
 
-    def setupGame(self, world, client):
+    def setupGame(self, game):
         '''
         Used for both replays and games.
         '''
         # Create a game interface.
-        self.gi = GameInterface(self.app, client, world)
+        self.gi = gi = GameInterface(self.app, game.world,
+                onDisconnectRequest=self._disconnectGame,
+                onConnectionLost=self.connectionLost)
+        game.addAgent(gi.eventPlug, gi.controller)
 
-        # Plug it to the netclient        
-        interface = InterfaceComponent(self.gi)
-        world.actionPlug.connectPlug(interface.plug)
-        self.gi.controller.connectPlug(client.orderPlug)
+    def _disconnectGame(self):
+        '''
+        Disconnected from the game.
+        '''
+        if self.netHandler is not None:
+            self.netHandler.disconnect()
+            self.netHandler = None
 
     def connectionFailed(self, reason):
         self.elements = [self.backdrop, self.startupInterface]
-        if not isinstance(reason.value, networkClient.UnableToConnect):
+        if True:
             print 'Unexpected failure in deferred: %s' % (reason.value,)
             reason.printTraceback()
             text = 'Internal error'
@@ -108,16 +119,15 @@ class Interface(framework.CompoundElement):
         d = ConnectionFailedDialog(self.app, text)
         d.Show()
 
-    def connectionLost(self, reason = None):
+    def connectionLost(self, reason=None):
         self.elements = [self.backdrop, self.startupInterface]
         if reason is not None:
-            if self.gi:
-                self.gi.connectionLost()
-                self.gi = None
+            self.gi = None
             d = ConnectionFailedDialog(self.app, reason)
             d.Show()
-        
+
     def connectToReplay(self, filename):
+        assert False, 'Replays are broken for now.'
         self.startupInterface.mainMenu()
 
         client = replays.makeClient(self.app, filename)
