@@ -20,7 +20,6 @@ from trosnoth.src.gui.framework.tabContainer import TabSize
 from trosnoth.src.gui.framework import prompt
 from trosnoth.src.gui.framework.elements import TextElement, TextButton
 from trosnoth.src.gui.framework import framework
-from trosnoth.src.trosnothgui.pregame import colours
 from trosnoth.src.gui.common import *
 from trosnoth.src.utils.unrepr import unrepr
 from trosnoth.src.network.networkDefines import defaultServerPort
@@ -52,6 +51,7 @@ class InternetTab(Tab, framework.TabFriendlyCompoundElement):
         
         self.tabOrder = [self.serverNameBox, self.portBox]
 
+        colours = app.theme.colours
         self.elements = [self.serverNameBox,
                          self.portBox,
                          self.button('connect',  self.connect,               (0, -40), 'midbottom'),
@@ -75,8 +75,11 @@ class InternetTab(Tab, framework.TabFriendlyCompoundElement):
             self.clear()
 
     def button(self, text, onClick, pos, anchor='topleft'):
-        pos = Location(AttachedPoint(ScaledSize(*pos),self.tabContainer._getTabRect, anchor), anchor)
-        result = TextButton(self.app, pos, text, self.startupInterface.font, colours.mainMenuColour, colours.white)
+        pos = Location(AttachedPoint(ScaledSize(*pos),
+                self.tabContainer._getTabRect, anchor), anchor)
+        colours = self.app.theme.colours
+        result = TextButton(self.app, pos, text, self.startupInterface.font,
+                colours.mainMenuColour, colours.white)
         result.onClick.addListener(lambda sender: onClick())
         return result
 
