@@ -15,38 +15,28 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-
-# TODO: check we use all these imports
-import sys
-from socket import *
-
 import pygame
-from twisted.internet import protocol, reactor, task
-from twisted.internet.error import CannotListenError
-from twisted.internet.protocol import DatagramProtocol
+from twisted.internet import reactor, task
 
 from trosnoth.src.utils.utils import timeNow
 from trosnoth.src.utils import logging
-from trosnoth.src.trosnothgui import defines
 from trosnoth.src.gui.framework import framework
-from trosnoth.src.gui.framework import prompt
 from trosnoth.src.gui.framework.hint import Hint
 import trosnoth.src.gui.framework.prompt as prompt
-from trosnoth.src.gui.framework.elements import PictureElement, TextElement, TextButton
+from trosnoth.src.gui.framework.elements import TextElement, TextButton
 from trosnoth.src.gui.framework.tab import Tab
 from trosnoth.src.gui.framework.checkbox import CheckBox
-from trosnoth.src.gui.framework.listbox import ListBox
 from trosnoth.src.gui.framework.dialogbox import DialogBox, DialogResult, DialogBoxAttachedPoint, OkBox
 from trosnoth.data import startupMenu
 
-from trosnoth.src.gui.common import *
+from trosnoth.src.gui.common import (Location, ScaledSize, ScaledArea,
+        ScaledLocation, SizedImage)
 
 from trosnoth.src.trosnothgui.pregame.imageRadioButton import ImageRadioButton, RadioButtonGroup
 from trosnoth.src.trosnothgui.pregame.promptImageRadioButton import PromptImageRadioButton
 
 from trosnoth.data import getPath
 
-#TODO: remove from networkDefines
 from trosnoth.src.network.networkDefines import maxTeamNameLength
 
 class ServerSetupTab(Tab):
@@ -69,7 +59,7 @@ class ServerSetupTab(Tab):
             self.tabContainer.renameTab('Start a Game', self)
 
     def setName(self, name):
-        self.toCreate.setName(name)        
+        self.toCreate.setName(name)
 
     def draw(self, screen):
         # Automatically update every half second, but only lazily in case it's
@@ -78,7 +68,7 @@ class ServerSetupTab(Tab):
         if self._lastUpdate < time - 0.5:
             self.setAppropriateElements()
             self._lastUpdate = time
-        
+
         super(ServerSetupTab, self).draw(screen)
 
 class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
@@ -118,7 +108,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.radio.add(customRadio)
 
         self.radio.selected(largeRadio)
-        self.radio.onSelectionChanged.addListener(self.setSelectionNumber)   
+        self.radio.onSelectionChanged.addListener(self.setSelectionNumber)
 
 
         colours = app.theme.colours
@@ -133,10 +123,10 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                                  ScaledLocation(720, 295),
                                  colours.headingColour),
                      TextElement(self.app, 'Time Limit:', font,
-                                 ScaledLocation(600, 360), 
+                                 ScaledLocation(600, 360),
                                  colours.headingColour),
                      TextElement(self.app, 'minutes', app.screenManager.fonts.smallNoteFont,
-                                 ScaledLocation(720, 435), 
+                                 ScaledLocation(720, 435),
                                  colours.headingColour),
                      ]
 
@@ -149,7 +139,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.serverInput.onClick.addListener(self.setFocus)
         self.serverInput.onTab.addListener(self.tabNext)
         self.serverInput.onEnter.addListener(self.startServer)
-        
+
         self.maxPlayersInput = prompt.InputBox(self.app, ScaledArea(600, 275, 100, 60),
                                            initValue = '10',
                                            font = font, maxLength = 3,
@@ -180,15 +170,15 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         else:
             self.warnForRecording = False
         del json
-        
-        self.recordReplayBox = CheckBox(self.app, ScaledLocation(600, 520),
-                                        text = 'Save Replay',
-                                        font = app.screenManager.fonts.smallNoteFont,
-                                        colour = colours.mainMenuColour,
-                                        initValue = not self.warnForRecording,
-                                        style='circle')
-        
-        self.recordReplayBox.onValueChanged.addListener(self.recordReplayButtonClicked)
+
+#        self.recordReplayBox = CheckBox(self.app, ScaledLocation(600, 520),
+#                                        text = 'Save Replay',
+#                                        font = app.screenManager.fonts.smallNoteFont,
+#                                        colour = colours.mainMenuColour,
+#                                        initValue = not self.warnForRecording,
+#                                        style='circle')
+
+#        self.recordReplayBox.onValueChanged.addListener(self.recordReplayButtonClicked)
         self.invisibleGameBox = CheckBox(self.app, ScaledLocation(800, 520),
                                            text = 'Invisible',
                                            font = app.screenManager.fonts.smallNoteFont,
@@ -205,12 +195,12 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                 'start', app.screenManager.fonts.hugeMenuFont,
                 colours.startButton, colours.white)
         self.startButton.onClick.addListener(self.startServer)
-        
+
         self.input = [self.serverInput, self.maxPlayersInput, self.gameDurationInput]
         self.enableGameDuration(True)
 
         self.elements = self.text + self.input + \
-                        [self.recordReplayBox,
+                        [#self.recordReplayBox,
                          self.invisibleGameBox,
                          self.noLimitBox,
                          self.invalidInputText,
@@ -220,7 +210,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                         ]
         self.setFocus(self.serverInput)
         self.lastSelection = -1
-        self.setSelectionNumber(0)   
+        self.setSelectionNumber(0)
 
     def recordReplayButtonClicked(self, sender):
         if self.warnForRecording and sender.value:
@@ -230,7 +220,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
             box.Show()
             self.warnForRecording = False
             sender.setValue(False)
-                         
+
     def setSelectionNumber(self, index):
         playerSizes = [10, 4, 6]
         timeLimits = [45, 10, 20]
@@ -241,10 +231,10 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
             chosenLimit = int(self.gameDurationInput.getValue())
             if (self.lastSelection == -1 or self.lastSelection == 3 or timeLimits[self.lastSelection] == chosenLimit) and index != 3:
                 self.gameDurationInput.setValue(str(timeLimits[index]))
-            
+
         if index != 3 and index != -1:
             self.lastSelection = index
-        
+
 
     def setName(self, name):
         #if self.serverInput.getValue() == '':
@@ -277,7 +267,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
             if self.gameDurationInput.hasFocus:
                 self.setFocus(self.maxPlayersInput)
         self.setTabOrder()
-            
+
 
     def startServer(self, sender):
         values = self.getValues()
@@ -316,7 +306,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         self.setBackColours()
 
         invalid = False
-        
+
         colours = self.app.theme.colours
         height = width = maxPlayers = duration = name = None
         if self.serverInput.value == '':
@@ -326,7 +316,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
             self.serverInput.setBackColour(colours.invalidDataColour)
         else:
             name = self.serverInput.value
-    
+
         if numPlayers[0] <= self.getInt(self.maxPlayersInput.value) <= numPlayers[1]:
             maxPlayers = self.getInt(self.maxPlayersInput.value)
         else:
@@ -335,7 +325,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                                     (numPlayers[0], numPlayers[1]))
                 invalid = True
             self.maxPlayersInput.setBackColour(colours.invalidDataColour)
-            
+
 
         if self.noLimitBox.value:
             duration = 0
@@ -345,8 +335,8 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
                 self.gameDurationInput.setBackColour(colours.invalidDataColour)
                 if not invalid:
                     invalid = True
-                    self.incorrectInput('Please set a duration for this game') 
-                
+                    self.incorrectInput('Please set a duration for this game')
+
         radioChoice = self.radio.getSelectedValue()
         if radioChoice is None:
             if not invalid:
@@ -356,12 +346,12 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         if invalid:
             return None
 
-                    
-        recordReplay = self.recordReplayBox.value
+
+        recordReplay = False#self.recordReplayBox.value
         invisible = self.invisibleGameBox.value
         width, height = radioChoice
 
-        
+
         return [name, width, height, maxPlayers, duration, recordReplay,
                 invisible]
 
@@ -369,7 +359,7 @@ class ServerDoesntExistMenu(framework.TabFriendlyCompoundElement):
         if value == '':
             return 0
         return int(value)
-        
+
 
     def incorrectInput(self, string):
         self.invalidInputText.setText(string)
@@ -416,7 +406,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
                                  ScaledLocation(70, 320),
                                  colours.headingColour),
                            TextElement(self.app, 'Game Mode:', font,
-                                 ScaledLocation(70, 400), 
+                                 ScaledLocation(70, 400),
                                  colours.headingColour)]
 
         # Dynamic text
@@ -483,7 +473,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         self.elements = self.staticText + self.dynamicText + self.input + self.buttons
 
         self.updateStatus()
-       
+
     def saveSettings(self):
         colours = self.app.theme.colours
         cmd = self.getCmdInterface()
@@ -498,7 +488,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         if blueName != self.blueTeamName:
             cmd.setTeamName(0, blueName)
 
-        redName = self.redTeamInput.getValue().strip() 
+        redName = self.redTeamInput.getValue().strip()
         if redName != self.redTeamName:
             cmd.setTeamName(1, redName)
 
@@ -521,7 +511,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         cmd = self.getCmdInterface()
         if cmd is not None:
             cmd.startGame()
-        
+
     def shutdownServer(self):
         self.shutdownConfirmationBox = ShutdownDialog(self.app)
         self.shutdownConfirmationBox.onClose.addListener(self._shutdownDlgClose)
@@ -529,14 +519,13 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
 
     def _shutdownDlgClose(self):
         if self.shutdownConfirmationBox.result == DialogResult.OK:
-            cmd = self.getCmdInterface()
-            if cmd is not None:
-                cmd.shutdown()
+            if self.app.server is not None:
+                self.app.server.shutdown()
 
     # The following three functions are all related: if a user edits the team names or gamde mode
     # but doesn't save them, they will get a certain amount of time before the inputs are reset to
     # those obtained from the server.
-    
+
     def inputEdit(self):
         if self.blueTeamInput.getValue().strip() != self.blueTeamName or \
             self.redTeamInput.getValue().strip() != self.redTeamName or \
@@ -593,8 +582,8 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
 
     def getCmdInterface(self):
         if self.app.server is not None:
-            return self.app.server.cmdInterface
-        
+            return self.app.server.game
+
         self.parent.setAppropriateElements()
         return None
 
@@ -604,7 +593,7 @@ class ServerAlreadyExistsMenu(framework.TabFriendlyCompoundElement):
         cmd = self.getCmdInterface()
         if cmd is None:
             return
-        
+
         currentGameState = cmd.getGameState()
 
         if currentGameState != 'P':
@@ -686,7 +675,7 @@ class ShutdownDialog(DialogBox):
                     app.screenManager.fonts.mediumMenuFont,
                     Location(DialogBoxAttachedPoint(self, ScaledSize(0,220),
                     'midtop'), 'midtop'), colours.invalidDataColour)]
-        
+
         font = app.screenManager.fonts.dialogButtonFont
 
         # OK Button
@@ -709,7 +698,7 @@ class ShutdownDialog(DialogBox):
     def ok(self):
         self.result = DialogResult.OK
         self.Close()
-        
+
     def cancel(self):
         self.result = DialogResult.Cancel
         self.Close()
