@@ -15,8 +15,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-from trosnoth.data import getPath, user
+from trosnoth.data import getPath, user, makeDirs
 from trosnoth.src.trosnothgui.defines import *
+import os
 
 def getName():
     nameFilename = getPath(user, 'name')
@@ -32,4 +33,12 @@ def writeName(name):
     nameFilename = getPath(user, 'name')
     nameFile = open(nameFilename , 'w')
     nameFile.write(name)
-    nameFile.close()
\ No newline at end of file
+    nameFile.close()
+
+def isFirstTime():
+    firstTimeFilename = getPath(user, 'first')
+    return not os.path.exists(firstTimeFilename)
+      
+def notFirstTime():
+    firstTimeFilename = getPath(user, 'first')
+    open(firstTimeFilename, 'w').close()
